/* prim_factors
 * Copyright (C) 2012 Joël Krähemann
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#include <stdlib.h>
#include <math.h>
#include <stdio.h>
#include <strings.h>

/**
 * is_prim:
 * @x ein Integer
 * Returns: 1 wenn es eine Primzahl ist, sonst 0.
 *
 * Ermittelt, ob die zahl @x eine Primzahl ist.
 */
int
is_prim(int x)
{
  int i;

  if (x == 2){
    return(1);
  }
  
  for (i = 3; i <= sqrt(x); i += 2){
    if (x % i == 0){
      return 0;
    }
  }

  return 1;
}

/**
 * prim_factors:
 * @x ein Integer
 * @factors ein Zeiger auf einen Zeiger, in dem die Faktoren gespeichert werden
 * @count ein Zeiger auf einen Integer, der die Anzahl Faktoren enthalten wird
 */
void
prim_factors(int x, int **x_factors, int *count)
{
  int *factors;
  int current_x;
  int i, n_factors;

  if(x < 2){
    *x_factors = NULL;
    *count = 0;
  }

  current_x = x;

  factors = NULL;
  n_factors = 0;

  for (i = 2; current_x != 0 && current_x - i != 0; ){
    if(!is_prim(i)){
      i++;
      continue;
    }

    if ((current_x % i) == 0){
      current_x /= i;

      n_factors++;
      factors = (int *) realloc(factors, n_factors * sizeof(int));
      factors[n_factors - 1] = i;
    }else{
      i++;
    }
  }

  if(current_x != 0){
    n_factors++;
    factors = (int *) realloc(factors, n_factors * sizeof(int));
    factors[n_factors - 1] = i;
  }

  *x_factors = factors;
  *count = n_factors;
}


/**
 * ggt:
 * @a ein Integer
 * @b ein Integer
 * Returns: Gibt den GGT zurück.
 *
 * Ermittelt den grössten gemeinsamen Teiler der beiden Zahlen @a und @b.
 */

int
ggt(int a, int b)
{
  int result;
  int *a_prims, *b_prims;
  int count_a, count_b;
  int i0, i1;

  if(a == b){
    return(a);
  }

  result = 1;

  prim_factors(a, &a_prims, &count_a);
  prim_factors(b, &b_prims, &count_b);

  i0 = 0;
  i1 = 0;

  while(i0 < count_a && i1 < count_b){
    if(a_prims[i0] == b_prims[i1]){
      result *= a_prims[i0];

      i1++;
    }else if(a_prims[i0] > b_prims[i1]){
      while(i1 < count_b && a_prims[i0] > b_prims[i1]){
	i1++;
      }
    }

    i0++;
  }

  if (result == 1 && (a != 1 || b != 1)){
    result = 1;
  }else if(result == a && (b % a != 0)){
    result = 1;
  }else if (result == b && (a % b != 0)){
    result = 1;
  }

  return(result);
}

/**
 * kgv:
 * @a ein Integer
 * @b ein Integer
 * Returns: Gibt das KGV zurück.
 *
 * Ermittelt das kleinste gemeinsame Vielfache der beiden Zahlen <paramref name="a"/> und <paramref name="b"/>.
 */
int
kgv(int a, int b){
  int result;
  int *a_prims, *b_prims;
  int count_a, count_b;
  int i0, i1;

  result = 1;

  prim_factors(a, &a_prims, &count_a);
  prim_factors(b, &b_prims, &count_b);

  i0 = 0;
  i1 = 0;

  while(i0 < count_a && i1 < count_b){
    if(a_prims[i0] == b_prims[i1]){
      result *= a_prims[i0];
      
      i0++;
      i1++;
    }else if(a_prims[i0] > b_prims[i1]){
      while(i1 < count_b && a_prims[i0] > b_prims[i1]){
	result *= b_prims[i1];

	i1++;
      }
    }else{
      result *= a_prims[i0];
      i0++;
    }
  }

  while(i0 != count_a){
    result *= a_prims[i0];

    i0++;
  }

  while(i1 < count_b){
    result *= b_prims[i0];

    i1++;
  }

  if(result == 1 && (a != 1 || b != 1)){
    result = 0;
  }

  return (result);
}

/**
 * write_line_and_read_int:
 * @message Die Nachricht, die vor der Zahlen eingabe erscheint (z.B. Zahl 1:).
 * Returns: Gibt eine int Zahl zurück.
 *
 * Gibt die Nachricht @message auf dem Bildschirm aus. Dann
 * liest es von der Standardeingabe bis ein Enter Schlag geschieht.
 * Die oberen zwei Befehle werden ausgeführt, bis eine gültig Integer Zahl eingegeben wurde.
 */
int
write_line_and_read_int(char *message)
{
  int a;
  int successfully_readed;

  a = 0;
  successfully_readed = 0;

  while(successfully_readed != 1){
    printf(message);
    successfully_readed = scanf("%d\0", &a);
  }

  return (a);
}


void
main(int argc, char **argv)
{
  char *command;
  int a, b;
  int result;
  int successfully_readed;

  printf("Hallo, das ist das Bruch Programm.\n\0");
  printf("Hiermit können Sie den GGT oder das KGV berechnen lassen.\n\0");

  command = malloc(1024 * sizeof(char));
  successfully_readed = 0;

  while(!0){
    bzero(command, 1024);
    
    printf("Geben Sie \"GGT\" oder \"KGV\" mit ihrer Tastatur ein.\n\0");
    printf("Sie werden dann aufgefordert zwei Zahlen einzugeben!\n\0");
    printf("Oder geben Sie \"beenden\" ein, um das Programm zu schliessen.\n\0");
    
    while(successfully_readed != 1){
      printf("> \0");
      successfully_readed = scanf("%s\0", command);
    }

    successfully_readed = 0;

    if(!strcmp("beenden\0", command)){
      break;
    }else if(!strcmp("KGV\0", command)){
      printf("Sie werden nun aufgefordert zwei Zahlen einzugeben Bestätigen Sie jede Zahl mit Enter.\n\0");

      a = write_line_and_read_int("Zahl 1: \0");
      b = write_line_and_read_int("Zahl 2: \0");
      
      result = kgv(a, b);
      
      printf("Das KGV von %d und %d beträgt: %d\n\0", a, b, result);
    }else if (!strcmp("GGT", command)){
      printf("Sie werden nun aufgefordert zwei Zahlen einzugeben Bestätigen Sie jede Zahl mit Enter.\n\0");
      
      a = write_line_and_read_int("Zahl 1: \0");
      b = write_line_and_read_int("Zahl 2: \0");

      result = ggt(a, b);
      
      printf("Der GGT von %d und %d beträgt: %d\n\0", a, b, result);
    }else{
      printf("Tut mir leid, habe Ihre Eingabe nicht verstanden.\n\0");
    }
  }
  
  printf("Aufwiedereingeben!\n\0");
}
